/***************************************************************************
                         chaussette.c  -  
                             -------------------
    begin                : Sun May  2004
    copyright            : (C) 2005 by Lozano Vincent
    email                : lozano@enise.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <stdio.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "daemon.h"
extern FILE* robot_log;


#define BACKLOG 5

#define __DEBUG 


int prepare_for_connections(int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  struct hostent *host;
  char nomserveur[128];
  int yes=1;

  /* nom de la machine sur laquelle tourne le programme */
  if ( gethostname (nomserveur,sizeof(nomserveur)) == -1 )
    {
      perror("server:gethostname");
      return -1;
    }
#ifdef __DEBUG
  fprintf(robot_log,"serveur : %s en attente...\n",nomserveur);
#endif

  /* adresse de la machine */
  if ( (host=gethostbyname(nomserveur)) == NULL )
    {
      perror("server:gethostbyname");
      return -2;
    }


  /* préparation de la chaussette du serveur */
  /* on précise le numéro de port, l'adresse IP */
  memset( (void*)&srv_socket_adr,0,sizeof(srv_socket_adr) );
  memcpy( (void*)&srv_socket_adr.sin_addr,host->h_addr,host->h_length );
  srv_socket_adr.sin_family=AF_INET;
  srv_socket_adr.sin_port=htons(portnum);

  /* obtention d'un descripteur */
  if ( (srv_socket=socket(PF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("server:socket");
      return -3;

    }
  if (setsockopt(srv_socket, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1) 
	{ 
    perror("setsockopt");
    exit(1);
  }

  /* on associe la chaussette à l'adresse IP/N° port du serveur */
  if ( bind (srv_socket, 
	     (struct sockaddr*)&srv_socket_adr, 
	     sizeof(struct sockaddr)) == -1 )
    {
      perror("server:bind");
      return -4;
    }

  /* on écoute */
  if ( listen(srv_socket,BACKLOG) == -1 )
    {
      perror("server:listen");
      return -5;
    }

  return srv_socket;
}

int connect_to_server_by_ip(char* ipserveur, int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  struct in_addr ip_adr;

  inet_aton(ipserveur,&ip_adr);

  /* préparation de la chaussette */
  /* on précise l'adresse du serveur et le numéro de port */
  memset((void*)&srv_socket_adr,0,sizeof(srv_socket_adr));

  srv_socket_adr.sin_addr=ip_adr;
  srv_socket_adr.sin_family=AF_INET;
  srv_socket_adr.sin_port=htons(portnum);

  /* obtention du descripteur */
  if ( (srv_socket=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("socket");
      return -1;
    }

  /* connexion au serveur */
  if ( connect(srv_socket, 
	       (struct sockaddr*)&srv_socket_adr, 
	       sizeof(struct sockaddr) ) == -1 )
    {
      perror("connect");
      return -2;
    }
#ifdef __DEBUG
  fprintf(robot_log,"connexion à %s\n",ipserveur);
#endif
  return srv_socket;
}


int connect_to_server_by_name(char* nomserveur, int portnum)
{
  int srv_socket;
  struct sockaddr_in srv_socket_adr;
  struct hostent *host;

  /* adresse du serveur dont on connaît le nom */
  if ( (host=gethostbyname(nomserveur)) == NULL )
    {
      perror("gethostbyname");
      return -1;
    }

  /* préparation de la chaussette */
  /* on précise l'adresse du serveur et le numéro de port */
  memset((void*)&srv_socket_adr,0,sizeof(srv_socket_adr));
  memcpy((void*)&srv_socket_adr.sin_addr,
	 host->h_addr, 
	 host->h_length);
  srv_socket_adr.sin_family=AF_INET;
  srv_socket_adr.sin_port=htons(portnum);

  /* obtention du descripteur */
  if ( (srv_socket=socket(AF_INET,SOCK_STREAM,0)) == -1 )
    {
      perror("socket");
      return -2;
    }

  /* connexion au serveur */
  if ( connect(srv_socket, 
	       (struct sockaddr*)&srv_socket_adr, 
	       sizeof(struct sockaddr) ) == -1 )
    {
      perror("connect");
      return -3;
    }
  return srv_socket;
}
extern int errno;

int get_connexion(int srv_sd,struct sockaddr_in *cltadr)
{
  int fromlen = sizeof(struct sockaddr_in);
  int clt_sd;

  /* on récupère l'adresse du client qui s'est connecté */
  if (  (clt_sd=accept(srv_sd,(struct sockaddr*)cltadr,&fromlen)) == -1 )
    {
      if (errno!=EINTR)
	perror("accept");
      return -1;
    }
#ifdef __DEBUG
  fprintf(robot_log,"connexion de : %s\n",inet_ntoa(cltadr->sin_addr));
#endif
  return clt_sd;
}

int send_message(int sd, char* msg)
{
  int sentcars;

  /* envoi du message */
  if ( (sentcars=send ( sd, msg, strlen(msg)+1,0 )) == -1 )
    {
      return -1;
    }
  return sentcars;
}
