/***************************************************************************
                         configLabjack.c  -  Configuration De La Labjack
                             -------------------
    begin                : 2005 08
    copyright            : (C) 2005 by Petit Benjamin
    email                : bpetit@enise.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/

/***************************************************************************
 *   Note: Made with the help of M. Wiertlewski for the structure of the configuration
 ***************************************************************************/
#include "configLabjack.h"
#include <stdlib.h>
#include <string.h>
#include "daemon.h"

ConfigLabjack *configLabjack = NULL;
extern FILE* robot_log;

ConfigLabjack *
config_labjack_load (const char *file)
{
  ConfigLabjack *config;
  char tmp[511];
  int indice;
  char def[511];


  fprintf(robot_log,"\n========= Chargement de la configuration de la labjack =========\nFichier: %s\n",
     file);

  if (file == NULL)		/*Si le fichier n'est pas spécifié */
    {
      file = "files/Labjack.config";
      fprintf(robot_log,"Remarque (ConfigLabjack/Load): Fichier non specifie, j'utilise %s\n",
	 file);
      fprintf(robot_log,"\n========= Fin de chargement de configuration de la labjack avec erreur =========\n");
    }

  if (strlen (file) > 255)	/*Si le nom du fichier est trop long */
    {
      fprintf(robot_log,"Erreur (ConfigLabjack/Load): Nom de fichier trop long\n");
      fprintf(robot_log,"\n========= Fin de chargement de configuration de la labjack avec erreur =========\n");
      return NULL;
    }

  else
    {
      config = (ConfigLabjack *) malloc (sizeof (ConfigLabjack));
      strcpy (config->file, file);

      if ((config->configFile = fopen (config->file, "r")) == NULL)
	{
	  fprintf(robot_log,"Erreur (ConfigLabjack/Load): Impossible d'ouvrir le fichier de configuration spécifié:\n%s\n",
	     config->file);
	  return NULL;
	}
      else
	{
	  while (!feof (config->configFile))
	    {
	      fscanf (config->configFile, "%s\n", tmp);

	      if ((tmp[0] != '/') && (tmp[1] != '/'))	//Si ce n'est pas un commentaire
		{
		  //Entreés analogiques
		  if (strstr (tmp, "AI") == tmp)
		    {
		      sscanf (tmp, "AI:%d=%s\n", &indice, def);
		      //fprintf(robot_log,"AI %d %s\n",indice,def);
		      if (indice > AI_MAX && indice < 0)
			fprintf(robot_log,"Erreur (ConfigLabjack/Load): Dépassement d indice pour AI %d\n",
			   indice);
		      else
			strcpy (config->ai_def[indice], def);
		    }

		  //Sorties analogiques
		  if (strstr (tmp, "AO") == tmp)
		    {
		      sscanf (tmp, "AO:%d=%s\n", &indice, def);
		      //fprintf(robot_log,"AO %d %s\n",indice,def);
		      if (indice > AO_MAX && indice < 0)
			fprintf(robot_log,"Erreur (ConfigLabjack/Load): Dépassement d indice pour AO %d\n",
			   indice);
		      else
			strcpy (config->ao_def[indice], def);
		    }

		  //Entrées/sorties digitales sur le carter
		  if (strstr (tmp, "IO") == tmp)
		    {
		      sscanf (tmp, "IO:%d=%s\n", &indice, def);
		      //fprintf(robot_log,"IO %d %s\n",indice,def);
		      if (indice > IO_MAX && indice < 0)
			fprintf(robot_log,"Erreur (ConfigLabjack/Load): Dépassement d indice pour IO %d\n",
			   indice);
		      else
			strcpy (config->io_def[indice], def);
		    }

		  //Entrées/sorties digitales sur le le D-Sub
		  if (strstr (tmp, "D") == tmp)
		    {
		      sscanf (tmp, "D:%d=%s\n", &indice, def);
		      //fprintf(robot_log,"D %d %s\n",indice,def);
		      if (indice > IO_MAX && indice < 0)
			fprintf(robot_log,"Erreur (ConfigLabjack/Load): Dépassement d indice pour D %d\n",
			   indice);
		      else
			strcpy (config->d_def[indice], def);
		    }
		  strcpy (def, "");	//nécessaire?? à voir
		}
	    }
	  fclose (config->configFile);
	}
      fprintf(robot_log,"Définition des Entrées Sorties chargées");
      fprintf(robot_log,"\n======= Fin de chargement de configuration de la labjack =======\n");
      /*
       * //ceci est le récapitulatif...
       * fprintf(robot_log,"\nRécapitulatif:\n");
       * for (indice=0;indice<=AI_MAX;indice++)
       * fprintf(robot_log,"AI %d -> %s\n",indice,config->ai_def[indice]);
       * for (indice=0;indice<=AO_MAX;indice++)
       * fprintf(robot_log,"AO %d -> %s\n",indice,config->ao_def[indice]);
       * for (indice=0;indice<=IO_MAX;indice++)
       * fprintf(robot_log,"IO %d -> %s\n",indice,config->io_def[indice]);
       * for (indice=0;indice<=D_MAX;indice++)
       * fprintf(robot_log,"D %d -> %s\n",indice,config->d_def[indice]);
       */
    }

  fprintf(robot_log,"\n");
  return config;
}

int
config_labjack_save (ConfigLabjack * config)
{
  int indice;

  if (config)
    {
      if ((config->configFile = fopen (config->file, "w")) == NULL)
	{
	  fprintf(robot_log,"Erreur (Configlabjack/Save): Impossible d'ouvrir le fichier de configuration spécifié:\n%s\n",
	     config->file);
	  return -1;
	}
      else
	{
	  for (indice = 0; indice < AI_MAX; indice++)
	    fprintf (config->configFile, "AI %d %s;\n",
		     indice, config->ai_def[indice]);
	  for (indice = 0; indice <= AO_MAX; indice++)
	    fprintf (config->configFile, "AO %d %s;\n",
		     indice, config->ao_def[indice]);
	  for (indice = 0; indice <= IO_MAX; indice++)
	    fprintf (config->configFile, "IO %d %s;\n",
		     indice, config->io_def[indice]);
	  for (indice = 0; indice <= D_MAX; indice++)
	    fprintf (config->configFile, "D %d %s;\n",
		     indice, config->d_def[indice]);

	  fclose (config->configFile);
	  return 0;
	}
    }
  else
    {
      fprintf(robot_log,"Erreur (ConfigLabjack/Save): Structure config inexistante");
      return -1;
    }
}

int
config_labjack_free (ConfigLabjack * config)
{
  if (config)
    {
      free (config);
      return 0;
    }
  else
    {
      fprintf(robot_log,"Erreur (ConfigLabjack/Free): Impossible de libérer le pointeur %p",
	 config);
      return -1;
    }
}


int config_labjack_cherche(ConfigLabjack * config,char* name,TypeIO* type,int* number)
{
	int i;
	
	//Si le nom n'est pas directement donnée en terme de labjack
	if ((strstr(name,"AI") == NULL)&&(strstr(name,"AO") == NULL)&&(strstr(name,"IO") == NULL)&&(strstr(name,"D") == NULL))
	{
		for (i=0;i<AI_MAX;i++)
		{
			if (!strcmp(name,config->ai_def[i]))
			{
				*type = ai;
				*number = i;
				return 0;
			}
		}			

		for (i=0;i<AO_MAX;i++)
		{
			if (!strcmp(name,config->ao_def[i]))
			{
				*type = ao;
				*number = i;
				return 0;
			}
		}		
		
		for (i=0;i<IO_MAX;i++)
		{
			if (!strcmp(name,config->io_def[i]))
			{
				*type = io;
				*number = i;
				return 0;
			}
		}		
		
		for (i=0;i<D_MAX;i++)
		{
			if (!strcmp(name,config->d_def[i]))
			{
				*type = d;
				*number = i;
				return 0;
			}
		}	
		return 1;		
	}
	else
	{
		if (!strncmp(name,"AI",2)) //si les 2 premiere lettre sont AI
		{
			*type = ai;
			*number = atoi(&(name[2]));
			return 0;
		}
		if (!strncmp(name,"AO",2)) //si les 2 premiere lettre sont AO
		{
			*type = ao;
			*number = atoi(&(name[2]));
			return 0;
		}
		
		if (!strncmp(name,"IO",2)) //si les 2 premiere lettre sont io
		{
			*type = io;
			*number = atoi(&(name[2]));  //numero deriere la chaine de caractere
			return 0;
		}
		if (!strncmp(name,"D",2)) //si les 2 premiere lettre sont D
		{
			*type = d;
			*number = atoi(&(name[2]));
			return 0;
		}
		return 1;
	}
	
	return 1;
}
