/***************************************************************************
                         configLabjack.h  -  Configuration De La Labjack
                             -------------------
    begin                : 2005 08
    copyright            : (C) 2005 by Petit Benjamin
    email                : bpetit@enise.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/

/***************************************************************************
 *   Note: Made with the help of M. Wiertlewski for the structure of the configuration
 ***************************************************************************/
#ifndef __CONF_LJ
#define __CONF_LJ
#include <stdio.h>

#define AI_MAX 8		//Entrées analogiques maximums
#define AO_MAX 2		//Sorties analogiques maximums
#define IO_MAX 3		//Entrées/sorties digitales maximums sur le carter
#define D_MAX 15		//Entrées/sorties digitales maximums sur le D-Sub

#define LJ_CONFIG_FILE "files/labjack.conf"


/************************Structures***********************/

typedef struct __ConfigLabjack
{
	/* Path du fichier de configuration */
	char file[255];
	FILE *configFile;

	// Attribut de configuration de la Labjack      
	// Nom correspondant aux différentes entrées/sorties: e.g AI0 -> TELEMETRE_AVANT_GAUCHE
	char ai_def[AI_MAX][255];	//Entrées analogiques
	char ao_def[AO_MAX][255];	//Sorties analogiques
	char io_def[IO_MAX][255];	//Entrées/sorties digitales sur le carter
	char d_def[D_MAX][255];	//Entrées/sorties digitales sur le D-Sub

} ConfigLabjack;

typedef enum 
{
	ai,
	ao,
	io,
	d
} TypeIO;

/************************Fonctions***********************/
ConfigLabjack* config_labjack_load (const char *file);
/*Charge la configuration à partir du fichier dans la structure
  Entree: 	Le fichier de configuration de la labjack
  Retour:	Une structure Configuration si le chargement a réussit
			-1 si le chargement a échoué
*/

int config_labjack_save (ConfigLabjack* config);
/* Sauve la structure dans le fichier precedement utilisé
  Entree:	La structure à sauver
  Retour:	0 si la libération a réussit
			-1 si la libération a échoué
*/

int config_labjack_free (ConfigLabjack* config);
/* Libère le pointeur config alloué par Configuration_load
  Entree:	La structure à sauver
  Retour:	0 si la libération a réussit
			-1 si la libération a échoué
*/
int config_labjack_cherche(ConfigLabjack* config,
		char* name,
		TypeIO* type,int* number);
/* Cherche la chaine dans Configuration et donne sa correspondance en terme de Labjack

  Entree:	La structure à utiliser
  			Nom du parametere a chercher
			*Type d'entrée ou de sortie
			*Numéro de l'entrée ou la sortie	
			
  Retour:	0 si la libération a réussit
			-1 si la libération a échoué
*/

#endif
