/***************************************************************************
                         daemon.c  -  
                             -------------------
    begin                : Sun May  2004
    copyright            : (C) 2005 by Lozano Vincent
    email                : lozano@enise.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>

void
log_message (char *filename, const char *action, const char *message)
{
	FILE *logfile;
	char str_date[150];
	time_t date = time (NULL);
	struct tm *tm_date = localtime (&date);

	logfile = fopen (filename, "a");
	if (!logfile)
		return;

	strftime (str_date, 150, "%F %T", tm_date);
	if (message)
		fprintf (logfile, "[%s] (%d) %s %s\n", str_date, getpid (),
			 action, message);
	else
		fprintf (logfile, "[%s] (%d) %s\n", str_date, getpid (),
			 action);
	fclose (logfile);
}



void
daemonize (int uid, char *lckfile, char *logfile, char *rundir)
{
	int i, lfp;
	char str[10];

	if (getppid () == 1)
		return;		/* already a daemon */

	i = fork ();
	if (i < 0)
		exit (1);	/* fork error */
	if (i > 0)
		exit (0);	/* parent exits */


	if (setuid (uid) == -1)
	{
		perror ("setuid");
		exit (1);
	}


	/* child (daemon) continues */
	if (setsid () == -1)	/* obtain a new process group */
	{
		perror ("setsid");
		exit (1);
	}


	for (i = getdtablesize (); i >= 0; --i)
		close (i);	/* close all descriptors */

	i = open ("/dev/null", O_RDWR);
	dup (i);
	dup (i);		/* handle standart I/O */


	umask (027);		/* set newly created file permissions */

	chdir (rundir);		/* change running directory */

	lfp = open (lckfile, O_RDWR | O_CREAT | O_EXCL, 0640);
	if (lfp < 0)
	{
		log_message (logfile,
			     "[daemonize] ouverture fichier lck : ",
			     strerror (errno));
		exit (1);
	}
	if (lockf (lfp, F_TLOCK, 0) < 0)
		exit (0);	/* can not lock */


	/* first instance continues */
	sprintf (str, "%d\n", getpid ());
	write (lfp, str, strlen (str));	/* record pid to lockfile */

/*   signal(SIGCHLD,SIG_IGN); /\* ignore child *\/ */
/*   signal(SIGTSTP,SIG_IGN); /\* ignore tty signals *\/ */
/*   signal(SIGTTOU,SIG_IGN); */
/*   signal(SIGTTIN,SIG_IGN); */
}
