/***************************************************************************
                         labjack.c  -  
                             -------------------
    begin                : Sun May  2004
    copyright            : (C) 2005 by Wiertlewski Michael
    email                : wiertlew@enise.fr
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/



// Fonction relatives à la labjack...
#include "labjack.h"
#include "ljackul.h"
#include <stdlib.h>
#include "daemon.h"

extern FILE* robot_log;
StateLabjack* lj_state = NULL;

int
labjack_init ()
{
	long id = -1;		// numéro d'identification de la Labjack: 
	// -1 siginifie la 1ere labjack trouvée
	float firmware;
	char ErrorString[255];
	// force l'initialisation de la Labjack
	InitLabjack ();
		
	// fait des tests pour vérifier l'existence de la Labjack
	firmware = GetFirmwareVersion (&id);

	if (id == -1)		// Labjack non détectée.. pas de
		// Labjack??
	{
		GetErrorString (id, ErrorString);
		fprintf(robot_log,"Erreur (Labjack): Pas de Labjack détéctée: %s\n",
			ErrorString);
		return (1);
	}
	else
		fprintf(robot_log,"Labjack: ID=%ld\n", id);

	
	if (firmware <= 0.0 || id > 512)	// autres erreurs...
	{
		GetErrorString (id - 512, ErrorString);
		fprintf(robot_log,"Erreur (Labjack): %s\n", ErrorString);
		return (1);
	}
	else
	{
//		int i;
		//Alloue la mémoire de la variable externe contenant l'etat des capteurs actionneurs.
		lj_state = (StateLabjack*) malloc(sizeof(StateLabjack));

		lj_state->id = id;
		lj_state->firmware = firmware;
		fprintf(robot_log,"Labjack: firmware version=%f\n",
			lj_state->firmware);

	}
	return (0);
}


int 
labjack_refresh()
{
	long channel1[4] = {0,1,2,3};
	long channel2[4] = {4,5,6,7};
	long gain[4] = {0,0,0,0};
	long error_code;
	int i;
	//Ecriture des parametere de sortie

	for (i=0;i<AI_MAX;i++)
		lj_state->ai_state[i] = 0;
	
	
	//Lecture des 4 premières entrée analogiques ainsi que des entrées sur le carter
	error_code = AISample(&lj_state->id,0,&lj_state->io_state,0,1
				,4,channel1,gain,0,&(lj_state->ai_overvoltage),lj_state->ai_state);
	
	if (error_code)
	{
		char ErrorString[255];
		GetErrorString (error_code, ErrorString);
		fprintf(robot_log,"Erreur (Labjack): %s\n", ErrorString);
		free(lj_state);
		return 1;
	}
	
	
	//lecture des 4 dernières entrée analogiques
	error_code = AISample(&lj_state->id,0,&lj_state->io_state,0,1
				,4,channel2,gain,0,&(lj_state->ai_overvoltage),&(lj_state->ai_state[4]));
	
	if (error_code)
	{
		char ErrorString[255];
		GetErrorString (error_code, ErrorString);
		printf ("Erreur (Labjack): %s\n", ErrorString);
		free(lj_state);
		return 1;
	}

	
	
	error_code = AOUpdate(&lj_state->id,0,
				lj_state->configD,lj_state->configIO,&lj_state->d_state,&lj_state->io_state,
				1,1,&lj_state->compteur,lj_state->ao_state[0],lj_state->ao_state[1]);
	
	if (error_code)
	{
		char ErrorString[255];
		GetErrorString (error_code, ErrorString);
		printf ("Erreur (Labjack): %s\n", ErrorString);
		free(lj_state);
		return 1;
	}
	return 0;
}



int
labjack_reset (long id)
{
	Reset (&id);
	if (id == -1)	// Labjack non détectée.. pas de
		// Labjack??
	{
		printf ("Erreur (Labjack): Pas de Labjack détéctée\n");
		if (lj_state)
			free(lj_state);
		
		return 1;
	}
	else if (id != lj_state->id)	// numéro d'identification
		// différent de celui détectée
		// à l'initialisation
	{
		printf ("Erreur (Labjack): Numéro de Labjack différent de l'initial\n");
		if (lj_state)
			free(lj_state);
		
		return 1;
	}
	else
		return 0;
}

int
labjack_stop (long id)
{
	fprintf(robot_log,"Labjack Fermée\n");
	free(lj_state);
	CloseAll (id);
	return 0;
}


void labjack_print()
{
	int i;
	fprintf(robot_log,"Io : %ld ,Etat E/S : %ld\n",lj_state->io_state,lj_state->configIO);
	fprintf(robot_log,"D  : %ld ,Etat E/S : %ld\n",lj_state->d_state,lj_state->configD);
	fprintf(robot_log,"\n");

	fprintf(robot_log,"Compteur : %ld\n",lj_state->compteur);
	fprintf(robot_log,"\n");

	for (i=0;i<AI_MAX;i++)
			fprintf(robot_log,"Ai %d : %f\n",i,lj_state->ai_state[i]);

	fprintf(robot_log,"\n");
	for (i=0;i<AO_MAX;i++)
			fprintf(robot_log,"Ao %d : %f\n",i,lj_state->ao_state[i]);
}
