/***************************************************************************
                         main.c  -  
                             -------------------
    begin                : 2005 07
    copyright            : (C) 2005 by Wiertlewski Michael
    email                : wiertlew@enise.fr
***************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This file is part of LabjackDaemon.
 *
 *   LabjackDaemon is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   Foobar is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Foobar; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 ***************************************************************************/


#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include "chaussette.h"
#include "daemon.h"
#include "configLabjack.h"
#include "labjack.h"


#define LOGFILE "/tmp/labjackd.log"
#define GetBit(byte,nb) byte & (1<<nb)


extern StateLabjack* lj_state;
extern ConfigLabjack *configLabjack;
int srv_socket;

void *thr_refresh_lj();
void *thr_dialogue();
int isole_bit(long int nb, int bit);


pthread_mutex_t mutex_lj_state = PTHREAD_MUTEX_INITIALIZER;
extern FILE* robot_log;

void
stoooooooooooop (int sig)
{
	switch (sig)
	{
	case SIGINT:
		fprintf(robot_log,"fermeture du socket\n");
		shutdown (srv_socket, SHUT_RDWR);
		if (lj_state)
			labjack_stop(lj_state->id);
		exit (1);
	}
}


int main(int argc, char** argv)
{

	pthread_t pid_refresh_lj;
	pthread_t pid_dialogue;

	robot_log = fopen(LOGFILE,"w");
	//verif des attributs	
	if (argc != 2)
	{
		fprintf (stderr, "usage %s <port>\n", argv[0]);
		argv[1] = "4006";
	}

	
	//toute les initialisations nécessaires
	labjack_init();  //initialise lj_state
	configLabjack = config_labjack_load (LJ_CONFIG_FILE);
	
	//création du thread de scrutation
	pthread_create(&pid_refresh_lj,NULL,thr_refresh_lj,NULL);
	
	
	//Début création server tcp
	struct sigaction action;

	int clt_socket;
	struct sockaddr_in adr_clt;
	int portnum;

	action.sa_handler = stoooooooooooop;
	sigaction (SIGINT, &action, NULL);

	portnum = atoi (argv[1]);

	if ((srv_socket = prepare_for_connections (portnum)) < 0)
	{
		fprintf (stderr, "prepare for connections %s\n",
			 strerror (errno));
		exit (1);
	}

	while(lj_state)
	{
		if ((clt_socket = get_connexion (srv_socket, &adr_clt)) != -1)//verifie si une connexion est établie
		{
			//Si la connextion est établie, on créé un thread de dialogue avec le socket.
			if (lj_state)
			{
				pthread_create(&pid_dialogue,NULL,thr_dialogue,&clt_socket);
			}
			else
			{
				send (clt_socket,"Labjack non branchée\n",
							strlen("Labjack non branchée\n"), 0);
				close(clt_socket);
			}			
		}
	}
	fprintf(robot_log,"\nLabjack Déconnecté. Fin de transmition\n");
	fprintf(stderr,"\nLabjack Déconnecté. Fin de transmition\n Voir %s",LOGFILE);

	//Fin création server tcp
	

	
	pthread_mutex_lock(&mutex_lj_state); 
	free(lj_state); //signal d'arret du thread de scrutation
	pthread_mutex_unlock(&mutex_lj_state);
	pthread_mutex_destroy(&mutex_lj_state); //detruit le mutex
	
	//Attent la fin de chaque thread
	pthread_join(pid_refresh_lj,NULL);
	
	free(configLabjack);
	
	return 0;
}

void *thr_refresh_lj()
{
	int reponse;
	while(lj_state)
	{
		pthread_mutex_lock(&mutex_lj_state); //bloque la structure pour les opérations d'écriture/lecture
		reponse = labjack_refresh();
		if (reponse)
			labjack_reset(lj_state->id);

		pthread_mutex_unlock(&mutex_lj_state); //débloque la situation.
		usleep(20);
	}
	return NULL;
}


void *thr_dialogue(void* socket)
{
	int *clt_socket = socket;

	char msg[128];
	
	do
	{
		if ((recv (*clt_socket, msg, sizeof (msg), 0)) == -1)
		{
			fprintf (stderr, "recv : %s",
			 strerror (errno));
			return NULL;
		}	
	
		fprintf(robot_log,"Message recu : %s\n",msg);
		if (!strncmp(msg,"SET ",4))
		{
			char object[255];
			int value;
			TypeIO type;
			int number;

			sscanf(&(msg[4]),"%s %d",object,&value);
			if (!config_labjack_cherche(configLabjack,object,&type,&number))
			{
				pthread_mutex_lock(&mutex_lj_state);
				send_message(*clt_socket,"Salut\n");
				switch (type)
				{
					case ao:
						if (number <= AO_MAX)
							lj_state->ao_state[number] = value;
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);
						break;
					case ai:
						send (*clt_socket,"Ceci est une entrée... Ecriture impossible\n",
										strlen("Ceci est une entrée... Ecriture impossible\n"), 0);
						break;
					case io:
						if (number <= IO_MAX)
						{
							if (isole_bit(lj_state->io_state,number) != value)
							{
								lj_state->configIO |= 1 << number; //configure l'io en sortie
								//Applique la valeur désirée :
								if (value == 0)
									lj_state->io_state &= ~(1 << number); 									
								else
									lj_state->io_state |= 1 << number; 	

								}
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);
						break;
					case d:
						if (number <= D_MAX)
						{
							if (isole_bit(lj_state->d_state,number) != value)
							{
								lj_state->configD |= 1 << number; //configure l'io en sortie
								//Applique la valeur désirée :
								if (value == 0)
									lj_state->d_state &= ~(1 << number); 
								else
									lj_state->d_state |= 1 << number; 	

							}
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);						
						break;				
				}
				pthread_mutex_unlock(&mutex_lj_state);
			}
			else send (*clt_socket,"Syntaxe incorrecte\n",
						strlen("Syntaxe incorrecte\n"), 0);
		
		}
		else if (!strncmp(msg,"GET ",4))
		{
			char object[255];
			TypeIO type;
			int number;
			char tmp[255];

			sscanf(&(msg[4]),"%s",object);
			
			if (!strcmp(object,"COMPTEUR"))
			{
				sprintf(tmp,"%ld\n",lj_state->compteur);
				send_message(*clt_socket,tmp);
				continue;
			}
			
			
			if (!config_labjack_cherche(configLabjack,object,&type,&number))
			{
				pthread_mutex_lock(&mutex_lj_state);
				switch (type)
				{
					case ai:
						if (number <= AI_MAX)
						{
							sprintf(tmp,"%lf\n",lj_state->ai_state[number]);
							send(*clt_socket,tmp,strlen(tmp),0);
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);	
						break;
					case ao:
						if (number <= AO_MAX)
						{
							sprintf(tmp,"%lf\n",lj_state->ao_state[number]);
							send(*clt_socket,tmp,strlen(tmp),0);
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);	
						break;
					case io:
						if (number <= IO_MAX)
						{
							sprintf(tmp,"%ld\n",GetBit(lj_state->io_state,number));
							send(*clt_socket,tmp,strlen(tmp),0);
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);	
						break;
					case d:
						if (number <= D_MAX)
						{
							sprintf(tmp,"%ld\n",GetBit(lj_state->d_state,number));
							send(*clt_socket,tmp,strlen(tmp),0);
						}
						else
							send (*clt_socket,"Sortie inexistante\n", 
									strlen("Sortie inexistante\n"), 0);	
						break;				
				}
				pthread_mutex_unlock(&mutex_lj_state);
			}
			else send (*clt_socket,"Syntaxe incorrecte\n",
						strlen("Syntaxe incorrecte\n"), 0);		
			
		}
		else 
		{
			if (!strncmp(msg,"EXIT",4))
				send (*clt_socket,"Syntaxe incorrecte\n",
						strlen("Syntaxe incorrecte\n"), 0);
		}
	
	} while (strncmp(msg,"EXIT",4));
	
	close(*clt_socket);
	fprintf(robot_log,"fin de thread\n");
	
	return NULL;
}



int isole_bit(long int nb, int bit)
{//bit de 0 à 31
 //exemple, si on entre comme valeur decimale 241, en binaire ça fait 11110001
 //si on souhaite lire le bit 4, il faut faire un & logique avec la valeur 10000 qui annulera les autres bit
 //ensuite on decale le resultat (11110001&10000=00010000) de 4 bit sur la droite et on lit la valeur (1)
	long int masque = 1;
	int i;
	
	//=exp(bit*log(2)); //2^bit
	for (i=0;i<bit;i++)
		masque *= 2;
		
	return ( (int)((nb & masque)/masque));
}
